@@ -8,11 +8,17 @@ int main(void) {
 
   igraph_rng_seed(igraph_rng_default(), 42);
 
-  igraph_erdos_renyi_game(&graph, IGRAPH_ERDOS_RENYI_GNM,
-                          num_vertices, num_edges,
-                          IGRAPH_UNDIRECTED, IGRAPH_NO_LOOPS);
+  igraph_erdos_renyi_game_gnm(
+    &graph, num_vertices, num_edges,
+    IGRAPH_UNDIRECTED, IGRAPH_NO_LOOPS
+  );
 
-  igraph_diameter(&graph, &diameter, NULL, NULL, NULL, NULL, IGRAPH_UNDIRECTED, /* unconn= */ true);
+  igraph_diameter(
+    &graph, &diameter,
+    /* from = */ NULL, /* to = */ NULL,
+    /* vertex_path = */ NULL, /* edge_path = */ NULL,
+    IGRAPH_UNDIRECTED, /* unconn= */ true
+  );
   printf("Diameter of a random graph with average degree %g: %g\n",
           2.0 * igraph_ecount(&graph) / igraph_vcount(&graph),
           (double) diameter);
