@@ -17,7 +17,7 @@
 
 #include "cliquer.h"
 
-#include "config.h"
+#include "config.h" /* IGRAPH_THREAD_LOCAL */
 
 /* Default cliquer options */
 IGRAPH_THREAD_LOCAL clique_options clique_default_options = {
@@ -1187,7 +1187,6 @@ igraph_error_t clique_unweighted_find_single(graph_t *g,int min_size,int max_siz
 	}
 	ASSERT(reorder_is_bijection(table,g->n));
 
-
 	if (unweighted_clique_search_single(table,min_size,g,opts) == 0) {
 		set_free(current_clique);
 		current_clique=NULL;
@@ -1222,6 +1221,7 @@ igraph_error_t clique_unweighted_find_single(graph_t *g,int min_size,int max_siz
 	}
 
     cleanreturn:
+	*clique = current_clique;
 
 	/* Free resources */
 	for (i=0; i < temp_count; i++)
@@ -1233,8 +1233,6 @@ igraph_error_t clique_unweighted_find_single(graph_t *g,int min_size,int max_siz
 	ENTRANCE_RESTORE();
 	entrance_level--;
 
-	*clique = current_clique;
-
 	return retval;
 }
 
