@@ -28,6 +28,7 @@
 #include "igraph_interface.h"
 #include "igraph_memory.h"
 #include "igraph_qsort.h"
+#include "igraph_random.h"
 
 #include "core/interruption.h"
 
@@ -35,8 +36,7 @@
 typedef struct {
     igraph_integer_t size;           /* Size of the community */
     igraph_real_t weight_inside;     /* Sum of edge weights inside community */
-    igraph_real_t weight_all;        /* Sum of edge weights starting/ending
-                                      in the community */
+    igraph_real_t weight_all;        /* Sum of edge weights starting/ending in the community */
 } igraph_i_multilevel_community;
 
 /* Global community list structure */
@@ -44,19 +44,18 @@ typedef struct {
     igraph_integer_t communities_no, vertices_no;  /* Number of communities, number of vertices */
     igraph_real_t weight_sum;              /* Sum of edges weight in the whole graph */
     igraph_i_multilevel_community *item;   /* List of communities */
-    igraph_vector_int_t *membership;           /* Community IDs */
-    igraph_vector_t *weights;        /* Graph edge weights */
+    igraph_vector_int_t *membership;       /* Community IDs */
+    igraph_vector_t *weights;              /* Graph edge weights */
 } igraph_i_multilevel_community_list;
 
 /* Computes the modularity of a community partitioning */
 static igraph_real_t igraph_i_multilevel_community_modularity(
-                                                              const igraph_i_multilevel_community_list *communities,
-                                                              const igraph_real_t resolution) {
-    igraph_real_t result = 0;
-    igraph_integer_t i;
+        const igraph_i_multilevel_community_list *communities,
+        const igraph_real_t resolution) {
+    igraph_real_t result = 0.0;
     igraph_real_t m = communities->weight_sum;
 
-    for (i = 0; i < communities->vertices_no; i++) {
+    for (igraph_integer_t i = 0; i < communities->vertices_no; i++) {
         if (communities->item[i].size > 0) {
             result += (communities->item[i].weight_inside - resolution * communities->item[i].weight_all * communities->item[i].weight_all / m) / m;
         }
@@ -96,7 +95,7 @@ static int igraph_i_multilevel_link_cmp(const void *a, const void *b) {
 /* removes multiple edges and returns new edge IDs for each edge in |E|log|E| */
 static igraph_error_t igraph_i_multilevel_simplify_multiple(igraph_t *graph, igraph_vector_int_t *eids) {
     igraph_integer_t ecount = igraph_ecount(graph);
-    igraph_integer_t i, l = -1, last_from = -1, last_to = -1;
+    igraph_integer_t l = -1, last_from = -1, last_to = -1;
     igraph_bool_t directed = igraph_is_directed(graph);
     igraph_vector_int_t edges;
     igraph_i_multilevel_link *links;
@@ -108,17 +107,17 @@ static igraph_error_t igraph_i_multilevel_simplify_multiple(igraph_t *graph, igr
     IGRAPH_CHECK_OOM(links, "Multi-level community structure detection failed.");
     IGRAPH_FINALLY(igraph_free, links);
 
-    for (i = 0; i < ecount; i++) {
+    for (igraph_integer_t i = 0; i < ecount; i++) {
         links[i].from = IGRAPH_FROM(graph, i);
         links[i].to = IGRAPH_TO(graph, i);
         links[i].id = i;
     }
 
-    igraph_qsort((void*)links, (size_t) ecount, sizeof(igraph_i_multilevel_link),
+    igraph_qsort(links, (size_t) ecount, sizeof(igraph_i_multilevel_link),
                  igraph_i_multilevel_link_cmp);
 
     IGRAPH_VECTOR_INT_INIT_FINALLY(&edges, 0);
-    for (i = 0; i < ecount; i++) {
+    for (igraph_integer_t i = 0; i < ecount; i++) {
         if (links[i].from == last_from && links[i].to == last_to) {
             VECTOR(*eids)[links[i].id] = l;
             continue;
@@ -178,9 +177,9 @@ static igraph_error_t igraph_i_multilevel_community_links(
         const igraph_i_multilevel_community_list *communities,
         igraph_integer_t vertex, igraph_vector_int_t *edges,
         igraph_real_t *weight_all, igraph_real_t *weight_inside, igraph_real_t *weight_loop,
-        igraph_vector_t *links_community, igraph_vector_t *links_weight) {
+        igraph_vector_int_t *links_community, igraph_vector_t *links_weight) {
 
-    igraph_integer_t i, n, last = -1, c = -1;
+    igraph_integer_t n, last = -1, c = -1;
     igraph_real_t weight = 1;
     igraph_integer_t to, to_community;
     igraph_integer_t community = VECTOR(*(communities->membership))[vertex];
@@ -188,7 +187,7 @@ static igraph_error_t igraph_i_multilevel_community_links(
 
     *weight_all = *weight_inside = *weight_loop = 0;
 
-    igraph_vector_clear(links_community);
+    igraph_vector_int_clear(links_community);
     igraph_vector_clear(links_weight);
 
     /* Get the list of incident edges */
@@ -199,7 +198,7 @@ static igraph_error_t igraph_i_multilevel_community_links(
     IGRAPH_CHECK_OOM(links, "Multi-level community structure detection failed.");
     IGRAPH_FINALLY(igraph_free, links);
 
-    for (i = 0; i < n; i++) {
+    for (igraph_integer_t i = 0; i < n; i++) {
         igraph_integer_t eidx = VECTOR(*edges)[i];
         weight = VECTOR(*communities->weights)[eidx];
 
@@ -228,10 +227,10 @@ static igraph_error_t igraph_i_multilevel_community_links(
     /* Sort links by community ID and merge the same */
     igraph_qsort((void*)links, (size_t) n, sizeof(igraph_i_multilevel_community_link),
                  igraph_i_multilevel_community_link_cmp);
-    for (i = 0; i < n; i++) {
+    for (igraph_integer_t i = 0; i < n; i++) {
         to_community = links[i].community;
         if (to_community != last) {
-            IGRAPH_CHECK(igraph_vector_push_back(links_community, to_community));
+            IGRAPH_CHECK(igraph_vector_int_push_back(links_community, to_community));
             IGRAPH_CHECK(igraph_vector_push_back(links_weight, links[i].weight));
             last = to_community;
             c++;
@@ -247,10 +246,10 @@ static igraph_error_t igraph_i_multilevel_community_links(
 }
 
 static igraph_real_t igraph_i_multilevel_community_modularity_gain(
-                                                                   const igraph_i_multilevel_community_list *communities,
-                                                                   igraph_integer_t community, igraph_integer_t vertex,
-                                                                   igraph_real_t weight_all, igraph_real_t weight_inside,
-                                                                   const igraph_real_t resolution) {
+        const igraph_i_multilevel_community_list *communities,
+        igraph_integer_t community, igraph_integer_t vertex,
+        igraph_real_t weight_all, igraph_real_t weight_inside,
+        const igraph_real_t resolution) {
     IGRAPH_UNUSED(vertex);
     return weight_inside -
            resolution * communities->item[community].weight_all * weight_all / communities->weight_sum;
@@ -268,18 +267,15 @@ static igraph_error_t igraph_i_multilevel_shrink(igraph_t *graph, igraph_vector_
     igraph_integer_t no_of_edges = igraph_ecount(graph);
     igraph_bool_t directed = igraph_is_directed(graph);
 
+    IGRAPH_ASSERT(igraph_vector_int_size(membership) == no_of_nodes);
+
     if (no_of_nodes == 0) {
         return IGRAPH_SUCCESS;
     }
 
-    if (igraph_vector_int_size(membership) < no_of_nodes) {
-        IGRAPH_ERROR("cannot shrink graph, membership vector too short",
-                     IGRAPH_EINVAL);
-    }
-
     IGRAPH_VECTOR_INT_INIT_FINALLY(&edges, 2*no_of_edges);
 
-    IGRAPH_CHECK(igraph_reindex_membership(membership, 0, NULL));
+    IGRAPH_CHECK(igraph_reindex_membership(membership, NULL, NULL));
 
     /* Create the new edgelist */
     IGRAPH_CHECK(igraph_get_edgelist(graph, &edges, /* bycol= */ false));
@@ -332,40 +328,24 @@ static igraph_error_t igraph_i_community_multilevel_step(
         igraph_real_t *modularity,
         const igraph_real_t resolution) {
 
-    igraph_integer_t i, j;
     igraph_integer_t vcount = igraph_vcount(graph);
     igraph_integer_t ecount = igraph_ecount(graph);
     igraph_real_t q, pass_q;
     /* int pass; // used only for debugging */
-    igraph_bool_t changed = false;
-    igraph_vector_t links_community;
+    igraph_bool_t changed;
+    igraph_vector_int_t links_community;
     igraph_vector_t links_weight;
     igraph_vector_int_t edges;
     igraph_vector_int_t temp_membership;
     igraph_i_multilevel_community_list communities;
-    igraph_vector_t node_order;
+    igraph_vector_int_t node_order;
 
-    /* Initial sanity checks on the input parameters */
-    if (igraph_is_directed(graph)) {
-        IGRAPH_ERROR("multi-level community detection works for undirected graphs only",
-                     IGRAPH_UNIMPLEMENTED);
-    }
-    if (igraph_vector_size(weights) < igraph_ecount(graph)) {
-        IGRAPH_ERROR("multi-level community detection: weight vector too short", IGRAPH_EINVAL);
-    }
-    if (igraph_vector_any_smaller(weights, 0)) {
-        IGRAPH_ERROR("weights must be positive", IGRAPH_EINVAL);
-    }
-    if (resolution < 0.0) {
-      IGRAPH_ERROR("The resolution parameter must be non-negative", IGRAPH_EINVAL);
-    }
-
-    IGRAPH_CHECK(igraph_vector_init_range(&node_order, 0, vcount));
-    IGRAPH_FINALLY(igraph_vector_destroy, &node_order);
-    IGRAPH_CHECK(igraph_vector_shuffle(&node_order));
+    IGRAPH_CHECK(igraph_vector_int_init_range(&node_order, 0, vcount));
+    IGRAPH_FINALLY(igraph_vector_int_destroy, &node_order);
+    igraph_vector_int_shuffle(&node_order);
 
     /* Initialize data structures */
-    IGRAPH_VECTOR_INIT_FINALLY(&links_community, 0);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&links_community, 0);
     IGRAPH_VECTOR_INIT_FINALLY(&links_weight, 0);
     IGRAPH_VECTOR_INT_INIT_FINALLY(&edges, 0);
     IGRAPH_VECTOR_INT_INIT_FINALLY(&temp_membership, vcount);
@@ -375,14 +355,14 @@ static igraph_error_t igraph_i_community_multilevel_step(
     communities.vertices_no = vcount;
     communities.communities_no = vcount;
     communities.weights = weights;
-    communities.weight_sum = 2 * igraph_vector_sum(weights);
+    communities.weight_sum = 2.0 * igraph_vector_sum(weights);
     communities.membership = membership;
     communities.item = IGRAPH_CALLOC(vcount, igraph_i_multilevel_community);
     IGRAPH_CHECK_OOM(communities.item, "Multi-level community structure detection failed.");
     IGRAPH_FINALLY(igraph_free, communities.item);
 
     /* Still initializing the communities data structure */
-    for (i = 0; i < vcount; i++) {
+    for (igraph_integer_t i = 0; i < vcount; i++) {
         VECTOR(*communities.membership)[i] = i;
         communities.item[i].size = 1;
         communities.item[i].weight_inside = 0;
@@ -390,7 +370,7 @@ static igraph_error_t igraph_i_community_multilevel_step(
     }
 
     /* Some more initialization :) */
-    for (i = 0; i < ecount; i++) {
+    for (igraph_integer_t i = 0; i < ecount; i++) {
         igraph_integer_t ffrom = IGRAPH_FROM(graph, i), fto = IGRAPH_TO(graph, i);
         igraph_real_t weight = 1;
 
@@ -405,16 +385,29 @@ static igraph_error_t igraph_i_community_multilevel_step(
     q = igraph_i_multilevel_community_modularity(&communities, resolution);
     /* pass = 1; */
 
+    RNG_BEGIN();
     do { /* Pass begin */
         igraph_integer_t temp_communities_no = communities.communities_no;
 
         pass_q = q;
-        changed = 0;
+        changed = false;
 
         /* Save the current membership, it will be restored in case of worse result */
         IGRAPH_CHECK(igraph_vector_int_update(&temp_membership, communities.membership));
 
-        for (i = 0; i < vcount; i++) {
+        /* Apply a random inversion to the node_order permutation vector to help escape
+         * rare situations of an infinite loop. A full re-shuffling of node_order would
+         * have a measurable performance impact, hence the single inversion.
+         * See https://github.com/igraph/igraph/issues/2650 for details. */
+        if (vcount > 1) {
+            igraph_integer_t i1 = RNG_INTEGER(0, vcount-1);
+            igraph_integer_t i2 = RNG_INTEGER(0, vcount-1);
+            igraph_integer_t tmp = VECTOR(node_order)[i1];
+            VECTOR(node_order)[i1] = VECTOR(node_order)[i2];
+            VECTOR(node_order)[i2] = tmp;
+        }
+
+        for (igraph_integer_t i = 0; i < vcount; i++) {
             /* Exclude vertex from its current community */
             igraph_real_t weight_all = 0;
             igraph_real_t weight_inside = 0;
@@ -435,7 +428,7 @@ static igraph_error_t igraph_i_community_multilevel_step(
             new_id = old_id;
 
             /* Update old community */
-            igraph_vector_int_set(communities.membership, ni, -1);
+            VECTOR(*communities.membership)[ni] = -1;
             communities.item[old_id].size--;
             if (communities.item[old_id].size == 0) {
                 communities.communities_no--;
@@ -448,9 +441,9 @@ static igraph_error_t igraph_i_community_multilevel_step(
             /* Find new community to join with the best modification gain */
             max_q_gain = 0;
             max_weight = weight_inside;
-            n = igraph_vector_size(&links_community);
+            n = igraph_vector_int_size(&links_community);
 
-            for (j = 0; j < n; j++) {
+            for (igraph_integer_t j = 0; j < n; j++) {
                 igraph_integer_t c = VECTOR(links_community)[j];
                 igraph_real_t w = VECTOR(links_weight)[j];
 
@@ -468,7 +461,7 @@ static igraph_error_t igraph_i_community_multilevel_step(
             /* debug("Added vertex %ld to community %ld (gain %lf).\n", ni, new_id, (double) max_q_gain); */
 
             /* Add vertex to "new" community and update it */
-            igraph_vector_int_set(communities.membership, ni, new_id);
+            VECTOR(*communities.membership)[ni] = new_id;
             if (communities.item[new_id].size == 0) {
                 communities.communities_no++;
             }
@@ -477,7 +470,7 @@ static igraph_error_t igraph_i_community_multilevel_step(
             communities.item[new_id].weight_inside += 2 * max_weight + weight_loop;
 
             if (new_id != old_id) {
-                changed++;
+                changed = true;
             }
         }
 
@@ -496,6 +489,7 @@ static igraph_error_t igraph_i_community_multilevel_step(
 
         IGRAPH_ALLOW_INTERRUPTION();
     } while (changed && (q > pass_q)); /* Pass end */
+    RNG_END();
 
     if (modularity) {
         *modularity = q;
@@ -504,7 +498,7 @@ static igraph_error_t igraph_i_community_multilevel_step(
     /* debug("Result Communities: %ld Modularity: %lf\n",
       communities.communities_no, (double) q); */
 
-    IGRAPH_CHECK(igraph_reindex_membership(membership, 0, NULL));
+    IGRAPH_CHECK(igraph_reindex_membership(membership, NULL, NULL));
 
     /* Shrink the nodes of the graph according to the present community structure
      * and simplify the resulting graph */
@@ -522,17 +516,17 @@ static igraph_error_t igraph_i_community_multilevel_step(
 
     /* We reuse the links_weight vector to store the old edge weights */
     IGRAPH_CHECK(igraph_vector_update(&links_weight, weights));
-    igraph_vector_fill(weights, 0);
+    igraph_vector_null(weights);
 
-    for (i = 0; i < ecount; i++) {
+    for (igraph_integer_t i = 0; i < ecount; i++) {
         VECTOR(*weights)[VECTOR(edges)[i]] += VECTOR(links_weight)[i];
     }
 
     igraph_free(communities.item);
-    igraph_vector_destroy(&links_community);
+    igraph_vector_int_destroy(&links_community);
     igraph_vector_destroy(&links_weight);
     igraph_vector_int_destroy(&edges);
-    igraph_vector_destroy(&node_order);
+    igraph_vector_int_destroy(&node_order);
     IGRAPH_FINALLY_CLEAN(5);
 
     return IGRAPH_SUCCESS;
@@ -541,16 +535,10 @@ static igraph_error_t igraph_i_community_multilevel_step(
 /**
  * \ingroup communities
  * \function igraph_community_multilevel
- * \brief Finding community structure by multi-level optimization of modularity.
+ * \brief Finding community structure by multi-level optimization of modularity (Louvain).
  *
- * This function implements the multi-level modularity optimization
- * algorithm for finding community structure, see
- * Blondel, V. D., Guillaume, J.-L., Lambiotte, R., &amp; Lefebvre, E. (2008). Fast
- * unfolding of communities in large networks. Journal of Statistical Mechanics:
- * Theory and Experiment, 10008(10), 6.
- * https://doi.org/10.1088/1742-5468/2008/10/P10008 for the details (preprint:
- * http://arxiv.org/abs/0803.0476). The algorithm is sometimes known as the
- * "Louvain" algorithm.
+ * This function implements a multi-level modularity optimization algorithm
+ * for finding community structure, sometimes known as the Louvain algorithm.
  *
  * </para><para>
  * The algorithm is based on the modularity measure and a hierarchical approach.
@@ -563,14 +551,24 @@ static igraph_error_t igraph_i_community_multilevel_step(
  * the modularity cannot be increased any more in a step.
  *
  * </para><para>
- * The resolution parameter \c gamma allows finding communities at different
+ * The resolution parameter \c γ allows finding communities at different
  * resolutions. Higher values of the resolution parameter typically result in
  * more, smaller communities. Lower values typically result in fewer, larger
  * communities. The original definition of modularity is retrieved when setting
- * <code>gamma=1</code>. Note that the returned modularity value is calculated using
+ * <code>γ=1</code>. Note that the returned modularity value is calculated using
  * the indicated resolution parameter. See \ref igraph_modularity() for more details.
  *
- * This function was contributed by Tom Gregorovic.
+ * </para><para>
+ * The original version of this function was contributed by Tom Gregorovic.
+ *
+ * </para><para>
+ * Reference:
+ *
+ * </para><para>
+ * Blondel, V. D., Guillaume, J.-L., Lambiotte, R., &amp; Lefebvre, E.:
+ * Fast unfolding of communities in large networks.
+ * Journal of Statistical Mechanics: Theory and Experiment, 10008(10), 6 (2008).
+ * https://doi.org/10.1088/1742-5468/2008/10/P10008
  *
  * \param graph       The input graph. It must be an undirected graph.
  * \param weights     Numeric vector containing edge weights. If \c NULL, every edge
@@ -596,18 +594,42 @@ static igraph_error_t igraph_i_community_multilevel_step(
  */
 
 igraph_error_t igraph_community_multilevel(const igraph_t *graph,
-                                const igraph_vector_t *weights,
-                                const igraph_real_t resolution,
-                                igraph_vector_int_t *membership,
-                                igraph_matrix_int_t *memberships, igraph_vector_t *modularity) {
+                                           const igraph_vector_t *weights,
+                                           const igraph_real_t resolution,
+                                           igraph_vector_int_t *membership,
+                                           igraph_matrix_int_t *memberships,
+                                           igraph_vector_t *modularity) {
 
     igraph_t g;
     igraph_vector_t w;
     igraph_vector_int_t m;
     igraph_vector_int_t level_membership;
     igraph_real_t prev_q = -1, q = -1;
-    igraph_integer_t i, level = 1;
+    igraph_integer_t level = 1;
     igraph_integer_t vcount = igraph_vcount(graph);
+    igraph_integer_t ecount = igraph_ecount(graph);
+
+    /* Initial sanity checks on the input parameters */
+    if (igraph_is_directed(graph)) {
+        IGRAPH_ERROR("Multi-level community detection works for undirected graphs only.",
+                     IGRAPH_UNIMPLEMENTED);
+    }
+    if (weights) {
+        if (igraph_vector_size(weights) != ecount) {
+            IGRAPH_ERROR("Weight vector length must agree with number of edges.", IGRAPH_EINVAL);
+        }
+        if (ecount > 0) {
+            igraph_real_t minweight = igraph_vector_min(weights);
+            if (minweight < 0) {
+                IGRAPH_ERROR("Weight vector must not be negative.", IGRAPH_EINVAL);
+            } else if (isnan(minweight)) {
+                IGRAPH_ERROR("Weight vector must not contain NaN values.", IGRAPH_EINVAL);
+            }
+        }
+    }
+    if (resolution < 0.0) {
+        IGRAPH_ERROR("The resolution parameter must be non-negative.", IGRAPH_EINVAL);
+    }
 
     /* Make a copy of the original graph, we will do the merges on the copy */
     IGRAPH_CHECK(igraph_copy(&g, graph));
@@ -626,7 +648,7 @@ igraph_error_t igraph_community_multilevel(const igraph_t *graph,
 
     if (memberships || membership) {
         /* Put each vertex in its own community */
-        for (i = 0; i < vcount; i++) {
+        for (igraph_integer_t i = 0; i < vcount; i++) {
             VECTOR(level_membership)[i] = i;
         }
     }
@@ -639,7 +661,7 @@ igraph_error_t igraph_community_multilevel(const igraph_t *graph,
         igraph_vector_clear(modularity);
     }
 
-    while (1) {
+    while (true) {
         /* Remember the previous modularity and vertex count, do a single step */
         igraph_integer_t step_vcount = igraph_vcount(&g);
 
@@ -652,9 +674,9 @@ igraph_error_t igraph_community_multilevel(const igraph_t *graph,
         }
 
         if (memberships || membership) {
-            for (i = 0; i < vcount; i++) {
+            for (igraph_integer_t i = 0; i < vcount; i++) {
                 /* Readjust the membership vector */
-                VECTOR(level_membership)[i] = VECTOR(m)[ VECTOR(level_membership)[i]];
+                VECTOR(level_membership)[i] = VECTOR(m)[ VECTOR(level_membership)[i] ];
             }
         }
 
@@ -682,14 +704,16 @@ igraph_error_t igraph_community_multilevel(const igraph_t *graph,
     if (modularity && igraph_vector_size(modularity) == 0) {
         igraph_vector_int_t tmp;
         igraph_real_t mod;
-        IGRAPH_VECTOR_INT_INIT_FINALLY(&tmp, vcount);
-        for (i = 0; i < vcount; i++) {
-            VECTOR(tmp)[i] = i;
-        }
+
+        IGRAPH_CHECK(igraph_vector_int_init_range(&tmp, 0, vcount));
+        IGRAPH_FINALLY(igraph_vector_int_destroy, &tmp);
+
         IGRAPH_CHECK(igraph_modularity(graph, &tmp, weights, resolution,
-                                       /* only undirected */ 0, &mod));
+                                       /* only undirected */ false, &mod));
+
         igraph_vector_int_destroy(&tmp);
         IGRAPH_FINALLY_CLEAN(1);
+
         IGRAPH_CHECK(igraph_vector_resize(modularity, 1));
         VECTOR(*modularity)[0] = mod;
     }
@@ -697,7 +721,7 @@ igraph_error_t igraph_community_multilevel(const igraph_t *graph,
     /* If we need the final membership vector, copy it to the output */
     if (membership) {
         IGRAPH_CHECK(igraph_vector_int_resize(membership, vcount));
-        for (i = 0; i < vcount; i++) {
+        for (igraph_integer_t i = 0; i < vcount; i++) {
             VECTOR(*membership)[i] = VECTOR(level_membership)[i];
         }
     }
