@@ -33,76 +33,76 @@ int test_graph(const char* name, const igraph_t* graph, const igraph_real_t* wei
         printf("UNWEIGHTED CASE:\n\n");
     }
 
-    igraph_global_efficiency(graph, &eff, NULL, 0);
+    igraph_global_efficiency(graph, &eff, NULL, IGRAPH_UNDIRECTED);
     printf("Global efficiency, undirected: %f\n", eff);
 
-    igraph_global_efficiency(graph, &eff, NULL, 1);
+    igraph_global_efficiency(graph, &eff, NULL, IGRAPH_DIRECTED);
     printf("Global efficiency, directed: %f\n", eff);
 
-    igraph_average_local_efficiency(graph, &eff, NULL, 0, IGRAPH_ALL);
+    igraph_average_local_efficiency(graph, &eff, NULL, IGRAPH_UNDIRECTED, IGRAPH_ALL);
     printf("Average local efficiency, undirected: %f\n", eff);
 
-    igraph_average_local_efficiency(graph, &eff, NULL, 1, IGRAPH_ALL);
+    igraph_average_local_efficiency(graph, &eff, NULL, IGRAPH_DIRECTED, IGRAPH_ALL);
     printf("Average local efficiency, directed, all neighbors: %f\n", eff);
 
-    igraph_average_local_efficiency(graph, &eff, NULL, 1, IGRAPH_IN);
+    igraph_average_local_efficiency(graph, &eff, NULL, IGRAPH_DIRECTED, IGRAPH_IN);
     printf("Average local efficiency, directed, in-neighbors: %f\n", eff);
 
-    igraph_average_local_efficiency(graph, &eff, NULL, 1, IGRAPH_OUT);
+    igraph_average_local_efficiency(graph, &eff, NULL, IGRAPH_DIRECTED, IGRAPH_OUT);
     printf("Average local efficiency, directed, out-neighbors: %f\n", eff);
 
     printf("\nLocal efficiency, undirected:\n");
-    igraph_local_efficiency(graph, &eff_vec, igraph_vss_all(), NULL, 0, IGRAPH_ALL);
+    igraph_local_efficiency(graph, &eff_vec, igraph_vss_all(), NULL, IGRAPH_UNDIRECTED, IGRAPH_ALL);
     print_vector(&eff_vec);
 
     printf("\nLocal efficiency, directed, all neighbors:\n");
-    igraph_local_efficiency(graph, &eff_vec, igraph_vss_all(), NULL, 1, IGRAPH_ALL);
+    igraph_local_efficiency(graph, &eff_vec, igraph_vss_all(), NULL, IGRAPH_DIRECTED, IGRAPH_ALL);
     print_vector(&eff_vec);
 
     printf("\nLocal efficiency, directed, in-neighbors:\n");
-    igraph_local_efficiency(graph, &eff_vec, igraph_vss_all(), NULL, 1, IGRAPH_IN);
+    igraph_local_efficiency(graph, &eff_vec, igraph_vss_all(), NULL, IGRAPH_DIRECTED, IGRAPH_IN);
     print_vector(&eff_vec);
 
     printf("\nLocal efficiency, directed, out-neighbors:\n");
-    igraph_local_efficiency(graph, &eff_vec, igraph_vss_all(), NULL, 1, IGRAPH_OUT);
+    igraph_local_efficiency(graph, &eff_vec, igraph_vss_all(), NULL, IGRAPH_DIRECTED, IGRAPH_OUT);
     print_vector(&eff_vec);
 
     if (weights_array) {
         igraph_vector_view(&weights, weights_array, igraph_ecount(graph));
         printf("\nWEIGHTED CASE:\n\n");
 
-        igraph_global_efficiency(graph, &eff, &weights, 0);
+        igraph_global_efficiency(graph, &eff, &weights, IGRAPH_UNDIRECTED);
         printf("Global efficiency, undirected: %f\n", eff);
 
-        igraph_global_efficiency(graph, &eff, &weights, 1);
+        igraph_global_efficiency(graph, &eff, &weights, IGRAPH_DIRECTED);
         printf("Global efficiency, directed: %f\n", eff);
 
-        igraph_average_local_efficiency(graph, &eff, &weights, 0, IGRAPH_ALL);
+        igraph_average_local_efficiency(graph, &eff, &weights, IGRAPH_UNDIRECTED, IGRAPH_ALL);
         printf("Average local efficiency, undirected: %f\n", eff);
 
-        igraph_average_local_efficiency(graph, &eff, &weights, 1, IGRAPH_ALL);
+        igraph_average_local_efficiency(graph, &eff, &weights, IGRAPH_DIRECTED, IGRAPH_ALL);
         printf("Average local efficiency, directed, all neighbors: %f\n", eff);
 
-        igraph_average_local_efficiency(graph, &eff, &weights, 1, IGRAPH_IN);
+        igraph_average_local_efficiency(graph, &eff, &weights, IGRAPH_DIRECTED, IGRAPH_IN);
         printf("Average local efficiency, directed, in-neighbors: %f\n", eff);
 
-        igraph_average_local_efficiency(graph, &eff, &weights, 1, IGRAPH_OUT);
+        igraph_average_local_efficiency(graph, &eff, &weights, IGRAPH_DIRECTED, IGRAPH_OUT);
         printf("Average local efficiency, directed, out-neighbors: %f\n", eff);
 
         printf("\nLocal efficiency, undirected:\n");
-        igraph_local_efficiency(graph, &eff_vec, igraph_vss_all(), &weights, 0, IGRAPH_ALL);
+        igraph_local_efficiency(graph, &eff_vec, igraph_vss_all(), &weights, IGRAPH_UNDIRECTED, IGRAPH_ALL);
         print_vector(&eff_vec);
 
         printf("\nLocal efficiency, directed, all neighbors:\n");
-        igraph_local_efficiency(graph, &eff_vec, igraph_vss_all(), &weights, 1, IGRAPH_ALL);
+        igraph_local_efficiency(graph, &eff_vec, igraph_vss_all(), &weights, IGRAPH_DIRECTED, IGRAPH_ALL);
         print_vector(&eff_vec);
 
         printf("\nLocal efficiency, directed, in-neighbors:\n");
-        igraph_local_efficiency(graph, &eff_vec, igraph_vss_all(), &weights, 1, IGRAPH_IN);
+        igraph_local_efficiency(graph, &eff_vec, igraph_vss_all(), &weights, IGRAPH_DIRECTED, IGRAPH_IN);
         print_vector(&eff_vec);
 
         printf("\nLocal efficiency, directed, out-neighbors:\n");
-        igraph_local_efficiency(graph, &eff_vec, igraph_vss_all(), &weights, 1, IGRAPH_OUT);
+        igraph_local_efficiency(graph, &eff_vec, igraph_vss_all(), &weights, IGRAPH_DIRECTED, IGRAPH_OUT);
         print_vector(&eff_vec);
     }
 
@@ -118,7 +118,7 @@ int test_ring(void) {
     igraph_t graph;
     const igraph_real_t weights_array[] = {1, 1, 1, 1};
 
-    igraph_ring(&graph, 4, IGRAPH_DIRECTED, /* mutual = */ 0, /* circular = */ 1);
+    igraph_ring(&graph, 4, IGRAPH_DIRECTED, /* mutual = */ false, /* circular = */ true);
     result = test_graph("Ring graph", &graph, weights_array);
     igraph_destroy(&graph);
 
@@ -132,7 +132,7 @@ int test_small_graph(void) {
     igraph_t graph;
     const igraph_real_t weights_array[] = {4, 4, 4, 3, 1, 5, 1, 2, 4, 5, 3, 5, 5, 4, 1, 1, 5, 4, 1, 1, 2, 1, 3, 5};
 
-    igraph_small(&graph, 13, /* directed= */ 1,
+    igraph_small(&graph, 13, IGRAPH_DIRECTED,
                  0,8, 1,3, 1,4, 1,5, 1,8, 1,10, 2,0, 2,1, 2,4, 3,5, 4,2, 4,7,
                  4,9, 5,3, 5,10, 6,7, 8,2, 8,3, 8,4, 8,9, 9,3, 9,4, 11,9, 11,3,
                  -1);
